/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: smbatvvrfy.c,v 1.5 2005/06/02 18:55:35 ca Exp $")
#include "sm/assert.h"
#include "sm/str.h"
#include "sm/io.h"
#include "sm/mta.h"
#include "sm/rfc2821.h"
#include "sm/batv.h"
#include "sm/batvcnf.h"

#if SM_USE_BATV
/*
**  SM_BATV_SIGN -- verify a BATV signature
**
**	Parameters:
**		sig -- signature
**		batv_keys -- BATV keys
**
**	Returns:
**		usual return code
*/

sm_ret_T
sm_batv_vrfy(sm_str_P address, batv_cnf_P batv_cnf)
{
	sm_ret_T ret;
	uint i, days_sig, days, key;
	bool found;
	time_t now;
	sm_str_P full, local, sig, domain;
	batv_T batv_sig_in, batv_sig_crt;
	sm_a2821_T a_addr;
	sm_a2821_P pa_addr;

	SM_REQUIRE(address != NULL);

	full = local = sig = domain = NULL;
	pa_addr = NULL;

	/* extract signature from address */
	A2821_INIT_RP(&a_addr, NULL);
	pa_addr = &a_addr;
	ret = t2821_scan((sm_rdstr_P) address, &a_addr, 0);
	if (sm_is_err(ret))
		goto error;
	ret = t2821_parse(&a_addr, 0);
	if (sm_is_err(ret))
		goto error;
	full = sm_str_new(NULL, MAXADDRLEN, MAXADDRLEN);
	if (full == NULL)
		goto enomem;
	domain = sm_str_new(NULL, MAXADDRLEN, MAXADDRLEN);
	if (domain == NULL)
		goto enomem;
	local = sm_str_new(NULL, MAXADDRLEN, MAXADDRLEN);
	if (local == NULL)
		goto enomem;
	sig = sm_str_new(NULL, 64, MAXADDRLEN);
	if (sig == NULL)
		goto enomem;

	/* CONF: BATV delimiter */
	ret = t2821_parts(&a_addr, batv_cnf->batvcnf_delim, false, local, sig,
			domain);
	if (sm_is_err(ret))
		goto error;
	if (ret == 0)
	{
		ret = sm_error_perm(SM_EM_BATV, SM_E_NOSIG);
		goto error;
	}
	a2821_free(pa_addr);
	pa_addr = NULL;

	/* remove signature from address */
	if (sm_is_err(ret = sm_str_put(full, '<')) ||
	    sm_is_err(ret = sm_str_cat(full, local)) ||
	    sm_is_err(ret = sm_str_put(full, '@')) ||
	    sm_is_err(ret = sm_str_cat(full, domain)) ||
	    sm_is_err(ret = sm_str_put(full, '>')))
		goto error;
	if (sm_str_getdata(sig) == NULL || sm_str_getdata(full) == NULL)
		goto enomem;

	/* scan signature */
	ret = sm_io_sscanf((const char *) sm_str_getdata(sig), "%1u%03u%06s",
			&key, &days_sig, batv_sig_in);
	if (ret != 3)
	{
		ret = sm_error_perm(SM_EM_BATV, SM_E_SYNTAX);
		goto error;
	}

	/* check expiration */
	now = time(NULL);
	days = now / (60 * 60 * 24);
	if (days_sig < (days % 1000))
	{
		ret = sm_error_perm(SM_EM_BATV, SM_E_SIGEXP);
		goto error;
	}

	/* search for key number */
	found = false;
	for (i = 0; i < batv_cnf->batvcnf_n; i++)
	{
		if (batv_cnf->batvcnf_keys[i].batv_key_number == key)
		{
			found = true;
			break;
		}
	}
	if (!found)
	{
		/* XXX better error code? */
		ret = sm_error_perm(SM_EM_BATV, SM_E_NOTFOUND);
		goto error;
	}

	/* sign address */
	ret = sm_batv_sign((const char *) sm_str_getdata(full), days_sig, key,
			batv_cnf, batv_sig_crt);
	if (sm_is_err(ret))
		goto error;

	/* cleanup */
	SM_STR_FREE(full);
	SM_STR_FREE(local);
	SM_STR_FREE(sig);
	SM_STR_FREE(domain);

	/* compare created signature with original data */
	if (strcmp(batv_sig_in, batv_sig_crt) == 0)
		return SM_SUCCESS;
	return sm_error_perm(SM_EM_BATV, EINVAL); /* XXX better error code? */

  enomem:
	ret = sm_error_temp(SM_EM_BATV, ENOMEM);
  error:
	SM_STR_FREE(full);
	SM_STR_FREE(local);
	SM_STR_FREE(sig);
	SM_STR_FREE(domain);
	a2821_free(pa_addr);
	return ret;
}
#endif /* SM_USE_BATV */
